% Description:
%    This code illustrates how to use PCA to decrease the dimension of data
%    set for the purpose of improving the  performance 
%    of Bayes rule in classifying data, relevant information is on 
%    https://kiwi.ecn.purdue.edu/rhea/index.php/PCA
% 
% Author:
%    Tian Zhou (zhou338@purdue.edu)
%    Intelligent System and Assistive Technology (ISAT) Lab
%    Industrial Engineering, Purdue University
%     
% Date: 
%     2014/3/30
% 


clear
clc
close all

% number of observations
N = 10000;
% parameters for multivariate Gaussian distribution
mu_1 = [2 2]; sigma_1 = [5 0; 0 5];
mu_2 = [-2 -2]; sigma_2 = [5 0; 0 5];

% class 1 and 2 are of equal probability
R = rand(N,1);
w = ones(size(R(:))) + (R(:) > 0.5);

% generate the original 2D data set
x = zeros(N,2);
for i = 1:N
    if (w(i) == 1)  
        x(i,:) = mvnrnd(mu_1, sigma_1);
    else
        x(i,:) = mvnrnd(mu_2, sigma_2);
    end
end

x1 = x(w == 1,:);
x2 = x(w == 2,:);

% plot the original data set
figure
plot(x1(:,1),x1(:,2),'xr',x2(:,1),x2(:,2),'ob');
legend('class 1', 'class 2');
title('Original data set in 2D')

% calculate the actual priori information, may not be 0.5 exactly
Prob_1 = sum(w(:) == 1)/N;
Prob_2 = sum(w(:) == 2)/N;

% use Bayes directly to do the classification in 2D
tic
C_1 = -1/2*log(det(sigma_1)) + log(Prob_1);
C_2 = -1/2*log(det(sigma_2)) + log(Prob_2); 
g = zeros(N,1);
for i = 1:N
g(i) = -1/2.*(x(i,:)-mu_1)*(sigma_1)^-1*(x(i,:)-mu_1)'... 
    + 1/2.*(x(i,:)-mu_2)*(sigma_2)^-1*(x(i,:)-mu_2)'...
    + C_1 - C_2;
end
w_predicted_bayes = ones(size(w)) + (g < 0);
error_bayes = x(w_predicted_bayes ~= w);
time = toc;
% figure
% hist(error_bayes,100);
% title('histogram of mis-classified values');
error_rate_bayes = sum(w_predicted_bayes ~= w) / N;
display(['Error rate using Bayes rule in 2D is: ' num2str(error_rate_bayes*100) '%']);
display(['Calculation time is: : ' num2str(time)]);

clock = tic;
% use PCA to reduce the data into 1D and then use Bayes rule for
% classification
x = x-repmat(mean(x),size(x,1),1);
cov_x = cov(x);

% find the eigenvectors and eigenvalues
[V,D] = eig(cov_x);

% extract diagonal of matrix as vector
D = diag(D);

% sort the variances in decreasing order
[~, rindices] = sort(-1*D);
V = V(:,rindices);
D = D(rindices);

% select the eigenvector corresponding to the larger eigenvalue as the
% principle vector
Feature_vec = V(:,1);

% project the original data set
x_PCA = x * Feature_vec;

x1 = x_PCA(w == 1,:);
x2 = x_PCA(w == 2,:);

time = toc(clock);
% plot the PCA-reduced data set
[N1,X1] = hist(x1,100);
[N2,X2] = hist(x2,100);
% hist(x1);
% h = findobj(gca,'Type','patch');
% set(h,'FaceColor',[0 .5 .5],'EdgeColor','w')
figure
bar(X1,N1,'r');
hold on;
bar(X2,N2,'b');
hold off;
% plot(1:length(x1),x1(:),'xr',1:length(x2),x2(:),'ob');
legend('class 1', 'class 2');
title('Histogram of PCA-reduced data set in 1D on most principle direction')

clock = tic;
% calculate the sample mean and std for Bayes
mu_1_PCA = mean(x1);
mu_2_PCA = mean(x2);
sigma_1_PCA = std(x1);
sigma_2_PCA = std(x2);

% Perform Bayes on this 1D data
C_1 = -log(sqrt(2*pi)*sigma_1_PCA) + log(Prob_1);
C_2 = -log(sqrt(2*pi)*sigma_2_PCA) + log(Prob_2); 
g = -1/2*((x_PCA-mu_1_PCA)/sigma_1_PCA).^2 + 1/2*((x_PCA-mu_2_PCA)/sigma_2_PCA).^2 + C_1 - C_2;
w_predicted_bayes = ones(size(w)) + (g < 0);
error_bayes = x(w_predicted_bayes ~= w);
% figure
% hist(error_bayes,100);
% title('histogram of mis-classified values');
error_rate_bayes = sum(w_predicted_bayes ~= w) / N;
time = time + toc(clock);
display(['Error rate using Bayes rule in 1D is: ' num2str(error_rate_bayes*100) '%']);
display(['Calculation time is: : ' num2str(time)]);